@@ -1,12 +1,10 @@
 """The names :py:class:`.render.Color`, :py:class:`.render.Material`,
 :py:class:`.render.Channel`, :py:class:`.render.ImageFormat`,
 :py:class:`.render.CallbackRenderer`, :py:class:`.render.BlockingRenderer`,
-:py:class:`.wrapper.NTracer`, :py:data:`.wrapper.CUBE`,
-:py:data:`.wrapper.SPHERE` and :py:func:`.kdtree_builder.build_composite_scene`
-are imported here for convenience."""
+:py:class:`.wrapper.NTracer`, :py:data:`.wrapper.CUBE` and
+:py:data:`.wrapper.SPHERE` are imported here for convenience."""
 
 
 from ntracer.render import Color,Material,Channel,ImageFormat,CallbackRenderer,BlockingRenderer
 from ntracer.wrapper import NTracer,CUBE,SPHERE
-from ntracer.kdtree_builder import build_composite_scene
 
