@@ -38,15 +38,15 @@ for t in [
     types[t[0]] = CType(*t)
 
 
-binary_op = """        FORCE_INLINE {wrap_type} operator{extra}({wrap_type} b) const {{
+binary_op = """        FORCE_INLINE {wrap_type} operator{extra}(const {wrap_type} &b) const {{
             return {intr}(this->data.p,b.data.p);
         }}
-        FORCE_INLINE {wrap_type} &operator{extra}=({wrap_type} b) {{
+        FORCE_INLINE {wrap_type} &operator{extra}=(const {wrap_type} &b) {{
             this->data.p = {intr}(this->data.p,b.data.p);
             return *this;
         }}
 """
-cmp_op = """        FORCE_INLINE mask operator{extra[0]}({wrap_type} b) const {{
+cmp_op = """        FORCE_INLINE mask operator{extra[0]}(const {wrap_type} &b) const {{
             return mask({intr}(this->data.p,b.data.p{extra[1]}));
         }}
 """
@@ -54,27 +54,15 @@ unary_meth = """        FORCE_INLINE {wrap_type} {extra}() const {{
             return {intr}(this->data.p);
         }}
 """
-reduce_meth = """        FORCE_INLINE {item_type} {extra}() const {{
-            return {intr}(this->data.p);
-        }}
-"""
-mask_meth = """        FORCE_INLINE {wrap_type} {extra}(mask m) const {{
+mask_meth = """        FORCE_INLINE {wrap_type} {extra}(const mask &m) const {{
             return {intr}(m.data,this->data.p);
         }}
 """
-pseudo_maskz_move = """        FORCE_INLINE {wrap_type} {extra}(mask m) const {{
-            return {intr}(reinterpret_cast<{base_type}>(m.data),this->data.p);
+pseudo_maskz_move = """        FORCE_INLINE {wrap_type} {extra}(const mask &m) const {{
+            return {intr}(impl::cast<{base_type}>(m.data),this->data.p);
         }}
 """
 
-have_vec_reduce_add = '        static constexpr bool has_vec_reduce_add = true;\n'
-reduce_add_meth = reduce_meth + have_vec_reduce_add
-reduce_add_meth_b = """        FORCE_INLINE single<{item_type}> reduce_add() const {{
-            auto r = {intr}(this->data.p,this->data.p);{more_steps}
-            return {intr}(r,r);
-        }}
-""" + have_vec_reduce_add
-
 load_meth = """        static FORCE_INLINE {wrap_type} {extra}(const {item_type} *source) {{
             return {intr}(reinterpret_cast<{iargs[0][type]}>(source));
         }}
@@ -90,12 +78,12 @@ set1_meth = """        static FORCE_INLINE {wrap_type} {extra}({item_type} x) {{
         }}
 """
 
-broadcast_meth = """        static FORCE_INLINE {wrap_type} repeat(single<{item_type}> x) {{
+broadcast_meth = """        static FORCE_INLINE {wrap_type} repeat(const single<{item_type}> &x) {{
             return {intr}(x.data);
         }}
 """
-shuffle_meth = """        static FORCE_INLINE {wrap_type} repeat(single<{item_type}> x) {{
-            return {intr}(x.data{extra},_MM_SHUFFLE(0,0,0,0));
+shuffle_meth = """        static FORCE_INLINE {wrap_type} repeat(const single<{item_type}> &x) {{
+            return {intr}(x.data{extra},0);
         }}
 """
 
@@ -104,30 +92,30 @@ setzero_meth = """        static FORCE_INLINE {wrap_type} {extra}() {{
         }}
 """
 
-cast_meth = """        explicit FORCE_INLINE {wrap_type}({wrap_type_src} b) {{
+cast_meth = """        explicit FORCE_INLINE {wrap_type}(const {wrap_type_src} &b) {{
             this->data.p = {intr}(b.data.p);
         }}
 """
-cast_meth_dec = """        explicit {wrap_type}({wrap_type_src} b);
+cast_meth_dec = """        explicit {wrap_type}(const {wrap_type_src} &b);
 """
-cast_meth_def = """    FORCE_INLINE {wrap_type}::{wrap_type}({wrap_type_src} b) {{
+cast_meth_def = """    FORCE_INLINE {wrap_type}::{wrap_type}(const {wrap_type_src} &b) {{
         this->data.p = {intr}(b.data.p);
     }}
 """
 
-binary_func = """    FORCE_INLINE {wrap_type} {extra}({wrap_type} a,{wrap_type} b) {{
+binary_func = """    FORCE_INLINE {wrap_type} {extra}(const {wrap_type} &a,const {wrap_type} &b) {{
         return {intr}(a.data.p,b.data.p);
     }}
 """
-cmp_func = """    FORCE_INLINE {wrap_type}::mask {extra[0]}({wrap_type} a,{wrap_type} b) {{
+cmp_func = """    FORCE_INLINE {wrap_type}::mask {extra[0]}(const {wrap_type} &a,const {wrap_type} &b) {{
         return {wrap_type}::mask({intr}(a.data.p,b.data.p{extra[1]}));
     }}
 """
 
-testz_func = """    FORCE_INLINE int {extra}({wrap_type} a,{wrap_type} b) {{
+testz_func = """    FORCE_INLINE int {extra}(const {wrap_type} &a,const {wrap_type} &b) {{
         return {intr}(a.data.p,b.data.p);
     }}
-    FORCE_INLINE int {extra}({wrap_type} a) {{
+    FORCE_INLINE int {extra}(const {wrap_type} &a) {{
         return {intr}(a.data.p,a.data.p);
     }}
 """
@@ -137,7 +125,7 @@ def print_mask_function(output,swidth,pwidth,func,args,body,fallback,explicit_ca
         swidth,
         pwidth,
         func,
-        ','.join('mask{0}_v_{1} {2}'.format(swidth,pwidth,chr(ord('a')+i)) for i in range(args))),end='',file=output)
+        ','.join('const mask{0}_v_{1} &{2}'.format(swidth,pwidth,chr(ord('a')+i)) for i in range(args))),end='',file=output)
     if explicit_cast:
         print('mask{0}_v_{1}('.format(swidth,pwidth),end='',file=output)
     
@@ -173,6 +161,22 @@ def generic_intrinsic_name(type,width,base,scalar=False):
         's' if scalar else 'p',
         type.code)
 
+def alt_int_intrinsic_name(type,width,base):
+    if type.float == FLOAT: return generic_intrinsic_name(type,width,base)
+    
+    return '{0}_{1}_si{2}'.format(mm_prefix(width),base,width)
+
+def letter_suffix(type):
+    if type is types['double']: return 'pd'
+    elif type is types['float']: return 'ps'
+    return 'si'
+
+def lane_code(type,lwidth):
+    return '{0}{1}x{2}'.format(
+        'f' if type.float == FLOAT else 'i',
+        type.size,
+        lwidth//type.size)
+
 def int_log2(x):
     r = 0
     x >>= 1
@@ -182,7 +186,7 @@ def int_log2(x):
     return r
 
 class FTransform(object):
-    def __init__(self,base_name,template,extra=None,support=FLOAT|INTEGER,prefer=None,scalar=False):
+    def __init__(self,base_name,template=None,extra=None,support=FLOAT|INTEGER,prefer=None,scalar=False):
         self.base_name = base_name
         self.template = template
         self.extra = extra or base_name
@@ -209,9 +213,9 @@ class FTransform(object):
         if self.prefer:
             if self.prefer.supported(type,width):
                 r = intrinsics.get(self.prefer.intrinsic(type,width))
-                if r: return r['tech']
+                if r: return r['cpuid']
             return self.prefer.antirequisite(type,width,intrinsics)
-        return None
+        return frozenset()
     
 class ShuffleTransform(FTransform):
     def __init__(self,prefer=None):
@@ -232,39 +236,96 @@ class BroadcastTransform(FTransform):
         super(BroadcastTransform,self).__init__(None,broadcast_meth)
     
     def intrinsic(self,type,width):
+        # GCC doesn't seem to have _mm_broadcastsd_pd, but _mm_broadcastsd_pd
+        # is an alias for _mm_movedup_pd which GCC does have
+        if type is types['double'] and width == 128: return '_mm_movedup_pd'
+        
         return generic_intrinsic_name(type,width,'broadcast' + {'int8_t':'b','int16_t':'w','int32_t':'d','int64_t':'q','float':'ss','double':'sd'}[type.name]) 
 
-class ReduceAddBTransform(FTransform):
-    def __init__(self,prefer=None):
-        super(ReduceAddBTransform,self).__init__('hadd',reduce_add_meth_b,prefer=prefer)
+class ReduceTransform(FTransform):
+    def supported(self,type,width):
+        return type.size >= 32
+    
+    @staticmethod
+    def shuffle1(type):
+        return '_MM_SHUFFLE2(0,1)' if type.size == 64 else '_MM_SHUFFLE(0,0,0,1)'
+    
+    def reduce_to_128(self,type,width,base):
+        if width == 512:
+            r = '''            auto tmp = {0}(data.p,_mm512_shuffle_{2}(data.p,data.p,_MM_SHUFFLE(0,0,3,2)));
+            auto r = _mm512_cast{1}512_{1}128({0}(tmp,_mm512_shuffle_{2}(tmp,tmp,_MM_SHUFFLE(0,0,0,1))));
+'''.format(
+                generic_intrinsic_name(type,512,base),
+                letter_suffix(type),
+                lane_code(type,128))
+            return r,'r',''
+        
+        if width == 256:
+            r = '            auto r = {0}(_mm256_cast{1}256_{1}128(data.p),{2}(data.p,{3}));\n'.format(
+                generic_intrinsic_name(type,128,base),
+                letter_suffix(type),
+                alt_int_intrinsic_name(type,256,'extractf128'),
+                self.shuffle1(type))
+            return r,'r',''
+        
+        assert width == 128
+        return '','data.p','auto '
     
     def output(self,type,width,intr):
-        ms = int_log2(width//type.size) - 2
-        if ms:
-            ms = '\n            r = {0}(r,r);'.format(intr['name']) * ms
-        else:
-            ms = ''
+        inter,invar,r_decl = self.reduce_to_128(type,width,self.base_name)
         
-        return self.template.format(
-            intr=intr['name'],
-            wrap_type=wrap_type(type,width),
-            item_type=type.name,
-            more_steps=ms)
+        action = lambda scalar: generic_intrinsic_name(type,128,self.base_name,type.float == FLOAT and scalar)
+        
+        assert type.size == 32 or type.size == 64
+        if type.size == 32:
+            # the floating-point shuffles operate on two vectors while the
+            # integer shuffles operate on one
+            extra = lambda: invar + ',' if type.float == FLOAT else ''
+        
+            shuf = generic_intrinsic_name(type,128,'shuffle')
+            inter += '            {0}r = {1}({2},{3}({2},{4}_MM_SHUFFLE(0,0,3,2)));\n'.format(
+                r_decl,action(False),invar,shuf,extra())
+            invar = 'r'
+            shuf = '{0}(r,{1}_MM_SHUFFLE(0,0,0,1))'.format(shuf,extra())
+        elif type.float == FLOAT:
+            shuf = '{0}({1},{1},_MM_SHUFFLE2(0,1))'.format(generic_intrinsic_name(type,128,'shuffle'),invar)
+        else:
+            shuf = '_mm_srli_si128({0},8)'.format(invar)
+        
+        return '''        FORCE_INLINE single<{0}> {1}() const {{
+{2}            return {3}({4},{5});
+        }}
+'''.format(type.name,self.extra,inter,action(True),invar,shuf)
+
+class ReduceAddTransform(ReduceTransform):
+    def __init__(self,prefer=None):
+        super(ReduceAddTransform,self).__init__('hadd',prefer=prefer)
+    
+    def intrinsic(self,type,width):
+        return super(ReduceAddTransform,self).intrinsic(type,128)
+    
+    def output(self,type,width,intr):
+        inter,invar,r_decl = self.reduce_to_128(type,width,'add')
+        
+        assert type.size == 32 or type.size == 64
+        if type.size == 32:
+            inter += '            {0}r = {1}({2},{2});\n'.format(r_decl,intr['name'],invar)
+            invar = 'r'
+        
+        return '''        FORCE_INLINE single<{0}> reduce_add() const {{
+{1}            return {2}({3},{3});
+        }}
+'''.format(type.name,inter,intr['name'],invar)
 
 class AltIFTransform(FTransform):
     def intrinsic(self,type,width):
-        if type.float == INTEGER:
-            return '{0}_{1}_si{2}'.format(
-                mm_prefix(width),
-                self.base_name,
-                width)
-        
-        return super(AltIFTransform,self).intrinsic(type,width)
+        return alt_int_intrinsic_name(type,width,self.base_name)
 
 class CmpFTransform(FTransform):
-    def __init__(self,op,template,code=None):
+    def __init__(self,op,template,code,ord):
         self.op = op
         self.code = code
+        self.ord = bool(ord)
         super(CmpFTransform,self).__init__(None,template)
         
     def intrinsic(self,type,width):
@@ -281,7 +342,7 @@ class CmpFTransform(FTransform):
     def output(self,type,width,intr):
         arg3 = ''
         if width >= 256 and type.float == FLOAT:
-            arg3 = ',_CMP_{0}_UQ'.format(self.code.upper())
+            arg3 = ',_CMP_{0}_{1}Q'.format(self.code.upper(),'UO'[self.ord])
 
         return self.template.format(
             intr=intr['name'],
@@ -297,6 +358,20 @@ class RepeatTransform(FTransform):
         if type is types['int64_t'] and width < 512:
             r += 'x'
         return r
+    
+    def output(self,type,width,intr):
+        r = self.template.format(
+            intr=intr['name'],
+            iargs=intr['parameters'],
+            wrap_type=wrap_type(type,width),
+            base_type=base_type(type,width),
+            item_type=type.name,
+            extra=self.extra)
+        
+        if intr['name'][-1] == 'x':
+            r = ifdef_support(['X64'],indent=1) + '\n' + r + '    #endif\n'
+        
+        return r
 
 class BlendTransformBase(FTransform):
     def __init__(self):
@@ -317,8 +392,8 @@ class BlendTransformBase(FTransform):
     def output(self,type,width,intr):
         params = self.base_params
         mparam = 'm.data'
-        if type.float == INTEGER and type.size >= 32:
-            mparam = 'reinterpret_cast<{0}>({1})'.format(base_type(type,width),mparam)
+        if type.float == INTEGER and type.size >= 32 and width < 512:
+            mparam = 'impl::cast<{0}>({1})'.format(base_type(type,width),mparam)
         
         if width < 512:
             params += ',' + mparam
@@ -343,7 +418,7 @@ class MaskSetTransform(BlendTransformBase):
 class BlendTransform(BlendTransformBase):
     # parameters are reversed to match "A ? B : C" syntax
     base_params = 'b.data.p,a.data.p'
-    template = '''    FORCE_INLINE {2} mask_blend(mask{0}_v_{1} m,{2} a,{2} b) {{
+    template = '''    FORCE_INLINE {2} mask_blend(const mask{0}_v_{1} &m,const {2} &a,const {2} &b) {{
         return {3}({4});
     }}
 '''
@@ -355,12 +430,9 @@ class CastTransform(object):
             ((width == 512 or src_width == 512) and type.size < 32))
     
     def intrinsic(self,type,width,src_width,inline):
-        code = 'si'
-        if type is types['double']: code = 'pd'
-        elif type is types['float']: code = 'ps'
         return '{0}_cast{1}{2}_{1}{3}'.format(
             mm_prefix(max(width,src_width)),
-            code,
+            letter_suffix(type),
             src_width,
             width)
     
@@ -380,19 +452,22 @@ class MixedFTransformAdapter(FTransform):
     def __init__(self,src_width,base):
         self.src_width = src_width
         self.base = base
-        super(MixedFTransformAdapter,self).__init__('',None)
+        super(MixedFTransformAdapter,self).__init__('')
+    
+    def supported(self,type,width):
+        return self.base.supported(type,width,self.src_width,True)
         
     def intrinsic(self,type,width):
-        return CastTransform().intrinsic(type,width,self.src_width,True)
+        return self.base.intrinsic(type,width,self.src_width,True)
     
     def output(self,type,width,intr):
-        return CastTransform().output(type,width,self.src_width,intr,True)
+        return self.base.output(type,width,self.src_width,intr,True)
 
 
 mixed_method_transforms = [
     CastTransform()]
 
-reduce_add = FTransform('reduce_add',reduce_add_meth)
+reduce_add = ReduceAddTransform()
 repeat_a = BroadcastTransform()
 maskz_mov = FTransform('maskz_mov',mask_meth,'zfilter')
 
@@ -404,21 +479,21 @@ method_transforms = [MixedFTransformAdapter(w,mmt) for w in widths for mmt in mi
     AltIFTransform('and',binary_op,'&'),
     AltIFTransform('or',binary_op,'|'),
     AltIFTransform('xor',binary_op,'^'),
-    CmpFTransform('==',cmp_op,'eq'),
-    CmpFTransform('!=',cmp_op,'neq'),
-    CmpFTransform('>',cmp_op,'gt'),
-    CmpFTransform('>=',cmp_op,'ge'),
-    CmpFTransform('<',cmp_op,'lt'),
-    CmpFTransform('<=',cmp_op,'le'),
+    CmpFTransform('==',cmp_op,'eq',True),
+    CmpFTransform('!=',cmp_op,'neq',True),
+    CmpFTransform('>',cmp_op,'gt',True),
+    CmpFTransform('>=',cmp_op,'ge',True),
+    CmpFTransform('<',cmp_op,'lt',True),
+    CmpFTransform('<=',cmp_op,'le',True),
     FTransform('sqrt',unary_meth,support=FLOAT),
     FTransform('rsqrt',unary_meth,support=FLOAT),
-    FTransform('abs',unary_meth),
+    FTransform('abs',unary_meth,support=INTEGER), # GCC doesn't seem to have _mm512_abs_ps or _mm512_abs_pd
     FTransform('ceil',unary_meth,support=FLOAT),
     FTransform('floor',unary_meth,support=FLOAT),
     reduce_add,
-    ReduceAddBTransform(prefer=reduce_add),
-    FTransform('reduce_max',reduce_meth),
-    FTransform('reduce_min',reduce_meth),
+    ReduceTransform('add',extra='reduce_add',prefer=reduce_add),
+    ReduceTransform('max',extra='reduce_max'),
+    ReduceTransform('min',extra='reduce_min'),
     AltIFTransform('load',load_meth),
     AltIFTransform('loadu',load_meth),
     AltIFTransform('store',store_meth),
@@ -441,10 +516,10 @@ function_transforms = [
     FTransform('min',binary_func),
     AltIFTransform('andnot',binary_func,'and_not'),
     AltIFTransform('testz',testz_func,'test_z'),
-    CmpFTransform('cmp_nlt',cmp_func,'nlt'),
-    CmpFTransform('cmp_nle',cmp_func,'nle'),
-    CmpFTransform('cmp_ngt',cmp_func,'ngt'),
-    CmpFTransform('cmp_nge',cmp_func,'nge'),
+    CmpFTransform('cmp_nlt',cmp_func,'nlt',False),
+    CmpFTransform('cmp_nle',cmp_func,'nle',False),
+    CmpFTransform('cmp_ngt',cmp_func,'ngt',False),
+    CmpFTransform('cmp_nge',cmp_func,'nge',False),
     BlendTransform()]
 
 
@@ -452,12 +527,12 @@ def data_args(args):
     return ['{0}.data'.format(chr(ord('a')+i)) for i in range(args)]
 
 def ideal_args(swidth,pwidth,args):
-    if swidth >= 32: args = ['reinterpret_cast<__m{0}i>({1})'.format(pwidth,a) for a in args]
+    if swidth >= 32: args = ['impl::cast<__m{0}i>({1})'.format(pwidth,a) for a in args]
     return ','.join(args)
 
 def avx512_mask_body(swidth,base,args):
     r = '_mm512_k{0}({1})'.format(base,','.join(data_args(args)))
-    if swidth != 32: r = 'static_cast<__mmask{0}>({1})'.format(512/swidth,r)
+    if swidth != 32: r = 'static_cast<__mmask{0}>({1})'.format(512//swidth,r)
     return r
 
 def common_mask_function(output,intrinsics,swidth,pwidth,type_req,suffix,name,intr_base):
@@ -470,9 +545,10 @@ def common_mask_function(output,intrinsics,swidth,pwidth,type_req,suffix,name,in
 
         body = '{0}_{1}_si{2}({3})'.format(mm_prefix(pwidth),intr_base,pwidth,ideal_args(swidth,pwidth,data_args(2)))
         
-        if intr['tech'] not in IMPLIED_REQ[type_req]:
+        req = intr['cpuid'] - type_req
+        if req:
             assert swidth >= 32
-            fallback = intr['tech'],'{0}_{1}_{2}(a.data,b.data)'.format(mm_prefix(pwidth),intr_base,suffix)
+            fallback = req,'{0}_{1}_{2}(a.data,b.data)'.format(mm_prefix(pwidth),intr_base,suffix)
         
     print_mask_function(output,swidth,pwidth,name,2,body,fallback)
 
@@ -498,9 +574,10 @@ def not_mask_function(output,intrinsics,swidth,pwidth,type_req,suffix):
             pwidth,
             [',_CMP_EQ_UQ',''][in_name])
         
-        if intr['tech'] not in IMPLIED_REQ[type_req]:
+        req = intr['cpuid'] - type_req
+        if req:
             assert swidth >= 32
-            fallback = [intr['tech'],'{0}_cmp{1}_{2}(a.data,a.data{3})'.format(
+            fallback = [req,'{0}_cmp{1}_{2}(a.data,a.data{3})'.format(
                 mm_prefix(pwidth),
                 ['','unord'][in_name],
                 suffix,
@@ -524,24 +601,26 @@ def xnor_mask_function(output,intrinsics,swidth,pwidth,type_req,suffix):
             ideal_args(swidth,pwidth,data_args(2)),
             [',_CMP_EQ_UQ',''][in_name])
         
-        if intr['tech'] not in IMPLIED_REQ[type_req]:
+        req = intr['cpuid'] - type_req
+        if req:
             assert swidth >= 32
-            fallback = intr['tech'],'!l_xor(a,b)'
+            fallback = req,'!l_xor(a,b)'
         
     print_mask_function(output,swidth,pwidth,'l_xnor',2,body,fallback,pwidth == 512)
 
 def mask_method_body(intr,intr_base,swidth,pwidth,type_req,args,suffix):
     body = '{0}({1})'.format(intr['name'],ideal_args(swidth,pwidth,args))
     
-    if intr['tech'] not in IMPLIED_REQ[type_req]:
+    req = intr['cpuid'] - type_req
+    if req:
         assert swidth >= 32
         body = '''
-    #ifdef SUPPORT_{0}
+    {0}
                 {1}
     #else
                 {2}_{3}_{4}({5})
     #endif
-                '''.format(macroize(intr['tech']),body,mm_prefix(pwidth),intr_base,suffix,','.join(args))
+                '''.format(ifdef_support(req),body,mm_prefix(pwidth),intr_base,suffix,','.join(args))
 
     return body
 
@@ -565,17 +644,18 @@ def any_mask_method(output,intrinsics,swidth,pwidth,type_req,suffix):
     # the default method is fine for AVX512
     if pwidth != 512:
         intr = intrinsics['{0}_testz_si{1}'.format(mm_prefix(pwidth),pwidth)]
-        always = intr['tech'] in IMPLIED_REQ[type_req]
+        req = intr['cpuid'] - type_req
         
-        if not always:
-            print(ifdef_support(intr['tech'],indent=1),sep='',file=output)
-            type_req = intr['tech']
+        if req:
+            print(ifdef_support(req,indent=1),sep='',file=output)
+            type_req = req | type_req
         
         print(
             '        FORCE_INLINE bool any() const {{\n            return {0} == 0;\n        }}'.format(
                 mask_method_body(intr,'testz',swidth,pwidth,type_req,['data','data'],suffix)),
             sep='',file=output)
-        if not always: print('    #endif\n',sep='',file=output)
+        
+        if req: print('    #endif\n',sep='',file=output)
 
 
 def macroize(x):
@@ -590,17 +670,21 @@ def base_type(type,width):
     elif type.float == INTEGER: suffix = 'i'
     return '__m{0}{1}'.format(width,suffix)
 
-def ifdef_support(req,antireq=None,indent=0):
+def ifdef_support(req,antireq=frozenset(),indent=0):
+    assert req or antireq
+    
     indent = '    '*indent
-    if req and antireq:
-        return '{0}#if defined(SUPPORT_{1}) && !defined(SUPPORT_{2})'.format(indent,macroize(req),macroize(antireq))
+    if len(req) + len(antireq) > 1:
+        parts = ['defined(SUPPORT_{0})'.format(macroize(r)) for r in req]
+        parts.extend('!defined(SUPPORT_{0})'.format(macroize(r)) for r in antireq)
+        return '{0}#if {1}'.format(indent,' && '.join(parts))
 
     extra = ''
     if not req:
         assert antireq
         extra = 'n'
         req = antireq
-    return '{0}#if{1}def SUPPORT_{2}'.format(indent,extra,macroize(req))
+    return '{0}#if{1}def SUPPORT_{2}'.format(indent,extra,macroize(list(req)[0]))
 
 def print_dependent(d_list,file,indent=0):
     for req,mt_list in d_list.items():
@@ -614,9 +698,11 @@ def print_dependent(d_list,file,indent=0):
 
 IMPLIED_REQ = {}
 rs = set()
-for r in ['SSE','SSE2','SSE3','SSSE3','SSE4.1','SSE4.2','AVX','AVX2','AVX-512']:
+for r in ['SSE','SSE2','SSE3','SSSE3','SSE4.1','SSE4.2','AVX','AVX2','AVX512F']:
     rs.add(r)
     IMPLIED_REQ[r] = frozenset(rs)
+for r in ['AVX512BW','AVX512CD','AVX512DQ','AVX512ER','AVX512IFMA52','AVX512PF','AVX512VBMI','AVX512VL']:
+    IMPLIED_REQ[r] = frozenset(rs.union([r]))
 del rs
 
 
@@ -627,14 +713,11 @@ mask_search = re.compile(r'^\s*//\s*\[\[\[mask-(\d+)-(\d+)(-outer|)\]\]\]\s*$')
 
 def generate(data_file,template_file,output_file):
     with open(data_file,'r') as input:
-        data = json.load(input)
-
-    intrinsics = {}
-    for item in data:
-        intrinsics[item['name']] = item
-
-    del data
-
+        intrinsics = json.load(input)
+    
+    for name,intr in intrinsics.items():
+        intr['cpuid'] = frozenset(intr['cpuid'])
+        intr['name'] = name
 
     template = open(template_file,'r')
     output = tempfile.NamedTemporaryFile(mode='w',dir=os.path.dirname(output_file),delete=False)
@@ -651,13 +734,15 @@ def generate(data_file,template_file,output_file):
                 outer = bool(m.group(3))
                 
                 if pwidth == 512:
-                    type_req = 'AVX-512'
+                    type_req = 'AVX512F'
                 elif pwidth == 256:
                     type_req = 'AVX2' if swidth < 32 else 'AVX'
                 else:
                     assert pwidth == 128
                     type_req = 'SSE' if swidth == 32 else 'SSE2'
                 
+                type_req = IMPLIED_REQ[type_req]
+                
                 if swidth == 64:
                     suffix = 'pd'
                 elif swidth == 32:
@@ -697,7 +782,7 @@ def generate(data_file,template_file,output_file):
                 else:
                     if width == 512:
                         assert type.size >= 32
-                        type_req = 'AVX-512'
+                        type_req = 'AVX512F'
                     elif type is types['float']:
                         type_req = {128:'SSE',256:'AVX'}[width]
                     elif type is types['double']:
@@ -708,18 +793,20 @@ def generate(data_file,template_file,output_file):
                         assert width == 128
                         type_req = 'SSE2'
                     
+                    type_req = IMPLIED_REQ[type_req]
+                    
                     dependent = {}
                     for mt in method_transforms:
                         if mt.supported(type,width):
                             intr = intrinsics.get(mt.intrinsic(type,width))
                             if intr:
-                                req = intr['tech']
+                                req = intr['cpuid'] - type_req
                                 antireq = mt.antirequisite(type,width,intrinsics)
 
-                                if antireq is None and req in IMPLIED_REQ[type_req]:
+                                if not (req or antireq):
                                     print(mt.output(type,width,intr),file=output.file)
-                                elif antireq is None or antireq not in IMPLIED_REQ[type_req]:
-                                    rs = (None if req in IMPLIED_REQ[type_req] else req,antireq)
+                                elif antireq.isdisjoint(type_req):
+                                    rs = (req,antireq)
                                     if rs not in dependent:
                                         dependent[rs] = []
                                     dependent[rs].append((mt,intr,type,width))
@@ -737,7 +824,7 @@ def generate(data_file,template_file,output_file):
                             
                             intr = intrinsics.get(ft.intrinsic(type,width))
                             if intr:
-                                rs = (intr['tech'],ft.antirequisite(type,width,intrinsics))
+                                rs = (intr['cpuid'],ft.antirequisite(type,width,intrinsics))
                                 if rs not in dependent:
                                     dependent[rs] = []
                                 dependent[rs].append((ft,intr,type,width))
